from setuptools import setup, find_packages

setup(
    name="elastic-logger-custom",
    version="1.0.7",
    install_requires=[
        "elasticsearch>=7.0.0,<8.0.0",
        'python-dotenv~=1.0.1',
    ],
    description="A custom logging handler for Elasticsearch",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
    packages=find_packages(),
)


import os

from .logger import LogHandler, ElasticLoggerConfig, create_logger

logger = None


def init_logger(config: ElasticLoggerConfig = None) -> None:
    """
    Initializes the global logger instance.

    :param config: ElasticLoggerConfig (optional). If not provided, it uses environment variables.
    """
    global logger  # Make sure the global variable is updated

    if config is None:
        hosts = os.getenv("ELASTICSEARCH_HOSTS")
        api_key = os.getenv("ELASTICSEARCH_API_KEY")
        index_name = os.getenv("ELASTICSEARCH_INDEX")
        application_name = os.getenv("APPLICATION_NAME")
        cert_path = os.getenv("ELASTICSEARCH_CERT_PATH", "./logger/elasticsearch-ca.crt")

        if not all([hosts, api_key, index_name, application_name]):
            raise ValueError("Missing required environment variables (ELASTICSEARCH_*)")

        config = ElasticLoggerConfig(
            hosts=hosts,
            api_key=api_key,
            index_name=index_name,
            application_name=application_name,
            cert_path=cert_path
        )

    logger = create_logger(config)  # Assign to global variable


try:
    init_logger()
except ValueError:
    pass

__all__ = ["logger", "init_logger", "ElasticLoggerConfig", "LogHandler"]



import datetime
import logging
import traceback
from dataclasses import dataclass

from elasticsearch import Elasticsearch


@dataclass
class ElasticLoggerConfig:
    hosts: str
    api_key: str
    index_name: str
    application_name: str
    cert_path: str = "./logger/elasticsearch-ca.crt"


def format_time(record: logging.LogRecord, datefmt: str = None) -> str:
    dt = datetime.datetime.fromtimestamp(record.created, datetime.timezone.utc)
    return dt.strftime(datefmt or "%Y-%m-%dT%H:%M:%SZ")


class LogHandler(logging.Handler):
    def __init__(self, config: ElasticLoggerConfig):
        super().__init__()
        self.client = Elasticsearch(
            hosts=[host.strip() for host in config.hosts.split(",")],
            api_key=config.api_key,
            verify_certs=False,
            ca_certs=config.cert_path
        )
        self.index_name = config.index_name
        self.application_name = config.application_name

        if not self.client.indices.exists(index=self.index_name):
            self.client.indices.create(
                index=self.index_name,
                body={
                    "settings": {"number_of_shards": 1, "number_of_replicas": 1},
                    "mappings": {
                        "properties": {
                            "timestamp": {"type": "date"},
                            "level": {"type": "keyword"},
                            "message": {"type": "text"},
                            "logger": {"type": "keyword"},
                            "filename": {"type": "keyword"},
                            "line": {"type": "integer"},
                            "function": {"type": "keyword"},
                            "application_name": {"type": "keyword"},
                            "stack_trace": {"type": "text"},
                        }
                    }
                }
            )

    def emit(self, record: logging.LogRecord):
        stack_trace = None
        if record.exc_info:
            stack_trace = ''.join(traceback.format_exception(*record.exc_info))

        log_entry = {
            "timestamp": format_time(record),
            "level": record.levelname,
            "message": record.getMessage(),
            "logger": record.name,
            "filename": record.pathname,
            "line": record.lineno,
            "function": record.funcName,
            "application_name": self.application_name,
            "stack_trace": stack_trace,
        }

        try:
            self.client.index(index=self.index_name, body=log_entry)
        except Exception as e:
            print(f"Failed to send log to Elasticsearch: {e}")


def create_logger(config: ElasticLoggerConfig) -> logging.Logger:
    logger = logging.getLogger("ElasticLogger")
    logger.setLevel(logging.INFO)

    if not logger.handlers:
        es_handler = LogHandler(config)
        logger.addHandler(es_handler)

        console_handler = logging.StreamHandler()
        console_handler.setFormatter(logging.Formatter("%(asctime)s - %(levelname)s - %(message)s"))
        logger.addHandler(console_handler)

    return logger
